#! /bin/bash

# cronspy - part of mail spy log htmlizer and summarizer
#
# A script designed to be run from cron as user mailspy (see crontab.spy)
# Launches htmlspy analysis and rotates the log
# Results end up as html documents ..
#
# Copyright (c) 2001, 2002 Andrew McGill and Leading Edge Business Solutions
# (South Africa).  
#
# Mailspy is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# This should bear some resemblance to what you read in rcmailspy ... :
RUNDIR="/var/run/mailspy"
MAILSPYSOCKET=$RUNDIR/milter
MAILSPYPIDFILE=$RUNDIR/pid
LOGFILE=/var/log/mailspy
MAILSPYUSER=mailspy
# Number of history entries to make
WEEKS=52
DAYS=$((WEEKS * 52))

OUTBASE=/usr/local/httpd/htdocs/mailspy
DIR=$OUTBASE/1
umask 022

function rotatedirs()
{

	# Rotate logfiles: assumes that all the files exist
	# /var/log/mailspy.5 -> /var/log/mailspy.tmp
	# /var/log/mailspy.4 -> /var/log/mailspy.5
	# /var/log/mailspy.3 -> /var/log/mailspy.4
	# /var/log/mailspy.2 -> /var/log/mailspy.3
	# /var/log/mailspy.1 -> /var/log/mailspy.2
	# /var/log/mailspy -> /var/log/mailspy.1
	# Blank mailspy.tmp
	# /var/log/mailspy.tmp -> /var/log/mailspy

	# we may be running as mailspy, so we can't delete the file
	mv -f $LOGFILE.$WEEKS $LOGFILE.tmp
	# Remove oldest dir
	rm -rf "$OUTBASE/$WEEKS"

	for (( a=WEEKS;a>1;a-- )) ; do 
		if ! [ -e $LOGFILE.$((a-1)) ] ; then
			touch $LOGFILE.$((a-1))
		fi
		mv -f $LOGFILE.$((a-1)) $LOGFILE.$a
		chown $MAILSPYUSER $LOGFILE.$a
		mv -f "$OUTBASE/$((a-1))" "$OUTBASE/$a"
	done
	mv -f $LOGFILE   $LOGFILE.1
	chown $MAILSPYUSER $LOGFILE.1
	> $LOGFILE.tmp
	mv -f $LOGFILE.tmp $LOGFILE
	chown $MAILSPYUSER $LOGFILE
	/etc/init.d/mailspy reload
}

function makeindexfile()
{
	{
	echo '<HTML>
	<HEAD><TITLE>Mail spy reports</TITLE></HEAD>
	<BODY>
	<H1>Mail spy reports</H1>
	<A href=current/index.html>Week-so-far report</A><BR>
	'
	( cd $OUTBASE; ls --sort=time */range.txt ; ) |
	while read INDEXFILE ; do
		DIRNAME=${INDEXFILE/range.txt/}
		echo " <A href=${DIRNAME}index.html>`cat $OUTBASE/$INDEXFILE`</A><BR>";
	done
	echo '</BODY></HTML>'
	} > "$OUTBASE/index.html"
}

function generatereport()
{
	# generatereport .1 1
	LOG=$LOGFILE$1;
	DIR="$OUTBASE/$2";
	mkdir -p "$DIR"
	nice /usr/local/bin/htmlspy.pl < $LOG "$DIR"
	chown -R mailspy $DIR 2>/dev/null
}

# Set up directories in the way that we expect them
function initdirs()
{
	mkdir -p $RUNDIR
	chown -R $MAILSPYUSER $RUNDIR
	for (( a=WEEKS;a>0;a-- )) ; do 
		if ! [ -e $LOGFILE.$a ] ; then
			touch $LOGFILE.$a
			chown $MAILSPYUSER $LOGFILE.$a
		fi
	done
	mkdir -p $OUTBASE
	chown $MAILSPYUSER $OUTBASE
}

case "$1" in 
	init)
		# secret initialisation thingy
		initdirs
		;;
	regenerate)
		# Redo ALL the reports
		for (( a=WEEKS;a>0;a-- )) ; do 
			echo "Generating $a"
			generatereport ".$a" "$a"
		done
		generatereport "" "current"
		makeindexfile
		;;
	makeindexfile)
		makeindexfile
		;;
	generatereport)
		# secret recovery thing (regenerate existing report)
		[ "$2" ] && generatereport ".$2" "$2";
		;;
	rotatedirs)
		rotatedirs
		makeindexfile
		;;
	weekly)
		rotatedirs
		generatereport ".1" "1";
		makeindexfile
		;;
	hourly)
		generatereport "" "current";
		;;
	daily)
		# Archive old procmailrc stuff
		cd ~mailspy/Mail
		ARCHNAME=`date +'%y-%m-%d-%a-%h'`
		mv -f today $ARCHNAME 2>/dev/null
		COUNT=0
		ls -c | while read FILE ; do
			[ $((++COUNT)) -gt $DAYS ] && rm $FILE ;
		done
		> discarded
		makeindexfile
		;;
	*)
		echo "Usage: $0 [weekly|daily|hourly|regenerate]"
		echo ""
		echo "  weekly and hourly generate html reports"
		echo "  daily rotates procmail logs files"
		echo "  regenerate regenerates all html pages"
		;;
esac

